:- [util].

write_spielfeld :-
  write_spieler,
  write_felder_geordnet,
  write_phase.

write_spieler :-
  write_ln('Hoefe:'),
  forall(spieler(_,Farbe,_),
         (write(Farbe),
		  atom_length(Farbe,FLength),
		  TabL is 7-FLength,
          tab(TabL),write_ritter_auf_hof(Farbe),
		  write(' | Reserve: '),
		  write_ritter_in_reserve(Farbe),
		  write('  '),
		  write_chips(Farbe),
		  nl
		 )
		).

write_ritter_auf_hof(Farbe) :-
  ritter_gesetzt(Farbe,Red,Green,Yellow,Blue,Pink),
  % rot
  (mehrheit(rot,Farbe)-> AR=[bold,fg(red)];AR=[fg(red)]),
  write_pieces(AR,r,Red),
  max_rot_gesetzt(_, MaxR),DiffR is MaxR-Red+1, tab(DiffR),
  % gruen
  (mehrheit(gruen, Farbe)-> AG=[bold,fg(green)];AG=[fg(green)]),
  write_pieces(AG,g,Green),
  max_gruen_gesetzt(_, MaxG),DiffG is MaxG-Green+1, tab(DiffG),
  % gelb
  (mehrheit(gelb, Farbe)-> AY=[bold,fg(yellow)];AY=[fg(yellow)]),
  write_pieces(AY,y,Yellow),
  max_gelb_gesetzt(_, MaxY),DiffY is MaxY-Yellow+1, tab(DiffY),
  % blau
  (mehrheit(blau, Farbe)-> AB=[bold,fg(blue)];AB=[fg(blue)]),
  write_pieces(AB,b,Blue),
  max_blau_gesetzt(_, MaxB),DiffB is MaxB-Blue+1, tab(DiffB),
  % pink
  (mehrheit(pink, Farbe)-> AP=[bold,fg(magenta)];AP=[fg(magenta)]),
  write_pieces(AP,p,Pink),
  max_pink_gesetzt(_, MaxP),DiffP is MaxP-Pink+2, tab(DiffP).

write_ritter_in_reserve(Farbe) :-
   reserve_ritter(Farbe,Red,Green,Yellow,Blue,Pink),
   write_pieces([fg(red)],r,Red),write_pieces([fg(green)],g,Green),
   write_pieces([fg(yellow)],y,Yellow),
   write_pieces([fg(blue)],b,Blue),write_pieces([fg(magenta)],p,Pink).

write_pieces(_,_,0).
write_pieces(AnsiOpts,C,N) :-
	N>0,
	ansi_format(AnsiOpts,'~w',[C]),
	N1 is N-1,
	write_pieces(AnsiOpts,C,N1).

write_felder :-
	nl, write_ln('Felder:'),
	forall(feld(Id,_,_),
	       write_feld(Id)
	      ).

write_felder_geordnet :-
	nl, write_ln('Felder:'),
	kleinstesFeld(Feld),
	write_felder_ord(Feld).

write_felder_ord(Anfang) :-
	feld(Anfang,_,Nachf),
	write_feld(Anfang),
	write_felder_ord(Anfang,Nachf).

write_felder_ord(Anfang,Anfang).

write_felder_ord(Anfang,Feld) :-
	Anfang \== Feld,
	feld(Feld,_,Nachf),
	write_feld(Feld),
	write_felder_ord(Anfang,Nachf).

write_feld(Id) :-
	feld(Id,_,_),
	(Id<10 -> write(0);true),
	write(Id),write(' '),
	(burgen(Id,BFarbe,BAnz) ->
	(write(BAnz),
	 write(' x '),
	 write('Burg'),
	 tab(1),
	 write(BFarbe),
	 tab(2),
	 write_ritter_auf_feld(Id)) ;
	write('keine Burg '),
	 write_ritter_auf_feld(Id)
	),
	(kaiser(Id) ->
	(tab(3),
	 write('Der Kaiser ist hier!'),
	 nl
	)
	;nl
	).

write_phase :- nl, writeln('Phase:'),
	aktueller_spieler(Spieler),
	(bietphase([Spieler|R]) -> (writeln('Bietphase'),
				   write('Aktueller Spieler: '),
				   writeln(Spieler),
				   write('Restliche: '),
				   writeln(R)
				  );
	phase(X) ->  (write('Ziehphase '),
		      phasen(X,Text),
		      writeln(Text),
		     write('Aktueller Spieler: '),
		      writeln(Spieler),
		      write('Restliche: '),
		      ziehphase([Spieler|R]),
		      writeln(R))
		     );true.

write_chips(Farbe) :-
	forall(member(X,[1,2,3,4,5]),
		   (noch_vorhandener_chip(Farbe,X)->
			write(X),write(' ');
			write('  ')
		   )
		  ).

%%%%%%%%%%
%% Praedikate, die Maxima berechnen
%%%%%%%%%%
max_rot_reserve(Farbe, Anz):-
	reserve_ritter(Farbe,Anz,_,_,_,_),
	\+ ( reserve_ritter(_,Anz2,_,_,_,_),Anz<Anz2).

max_gruen_reserve(Farbe, Anz):-
	reserve_ritter(Farbe,_,Anz,_,_,_),
	\+ ( reserve_ritter(_,_,Anz2,_,_,_),Anz<Anz2).

max_gelb_reserve(Farbe, Anz):-
	reserve_ritter(Farbe,_,_,Anz,_,_),
	\+ ( reserve_ritter(_,_,_,Anz2,_,_),Anz<Anz2).

max_blau_reserve(Farbe, Anz):-
	reserve_ritter(Farbe,_,_,_,Anz,_),
	\+ ( reserve_ritter(_,_,_,_,Anz2,_),Anz<Anz2).

max_pink_reserve(Farbe, Anz):-
	reserve_ritter(Farbe,_,_,_,_,Anz),
	\+ ( reserve_ritter(_,_,_,_,_,Anz2),Anz<Anz2).

max_rot_gesetzt(Farbe, Anz):-
	ritter_gesetzt(Farbe,Anz,_,_,_,_),
	\+ ( ritter_gesetzt(_,Anz2,_,_,_,_),Anz<Anz2).

max_gruen_gesetzt(Farbe, Anz):-
	ritter_gesetzt(Farbe,_,Anz,_,_,_),
	\+ ( ritter_gesetzt(_,_,Anz2,_,_,_),Anz<Anz2).

max_gelb_gesetzt(Farbe, Anz):-
	ritter_gesetzt(Farbe,_,_,Anz,_,_),
	\+ ( ritter_gesetzt(_,_,_,Anz2,_,_),Anz<Anz2).

max_blau_gesetzt(Farbe, Anz):-
	ritter_gesetzt(Farbe,_,_,_,Anz,_),
	\+ ( ritter_gesetzt(_,_,_,_,Anz2,_),Anz<Anz2).

max_pink_gesetzt(Farbe, Anz):-
	ritter_gesetzt(Farbe,_,_,_,_,Anz),
	\+ ( ritter_gesetzt(_,_,_,_,_,Anz2),Anz<Anz2).


write_ritter_auf_feld(Feld) :-
  ritter_auf_feld(Feld,Red,Green,Yellow,Blue,Pink),
  write_pieces([fg(red)],r,Red),
  write_pieces([fg(green)],g,Green),
   write_pieces([fg(yellow)],y,Yellow),
   write_pieces([fg(blue)],b,Blue),
   write_pieces([fg(magenta)],p,Pink).

