
:- dynamic(spielfeld/1).

% Zugriff auf die einzelnen Relationen
spieler(Name,Farbe,Burgen) :-
	spielfeld(X),member(spieler(Name,Farbe,Burgen),X).
reserve_ritter(ID,Rot,Gruen,Gelb,Blau,Pink) :-
	spielfeld(X),member(reserve_ritter(ID,Rot,Gruen,Gelb,Blau,Pink),X).
ritter_gesetzt(ID,Rot,Gruen,Gelb,Blau,Pink) :-
	spielfeld(X),member(ritter_gesetzt(ID,Rot,Gruen,Gelb,Blau,Pink),X).
noch_vorhandener_chip(ID,Chip) :-
	spielfeld(X),member(noch_vorhandener_chip(ID,Chip),X).
aktuelles_gebot(ID,Gebot) :-
	spielfeld(X),member(aktuelles_gebot(ID,Gebot),X).
mehrheit(Farbe,ID) :-
	spielfeld(X),member(mehrheit(Farbe,ID),X).
feld(Feld,Vorg,Nach) :-
	spielfeld(X),member(feld(Feld,Vorg,Nach),X).
burgen(Feld,Besitzer,Burgen) :-
	spielfeld(X),member(burgen(Feld,Besitzer,Burgen),X).
ritter_auf_feld(Feld,Rot,Gruen,Gelb,Blau,Pink) :-
	spielfeld(X),member(ritter_auf_feld(Feld,Rot,Gruen,Gelb,Blau,Pink),X).
kaiser(Feld) :-
	spielfeld(X),member(kaiser(Feld),X).
zugreihenfolge(ZugListe) :-
	spielfeld(X),member(zugreihenfolge(ZugListe),X).
bietphase(BietListe) :-
	spielfeld(X),member(bietphase(BietListe),X).
ziehphase(ZiehListe) :-
	spielfeld(X),member(ziehphase(ZiehListe),X).
phase(Phase) :-
	spielfeld(X),member(phase(Phase),X).
gewinner(GewinnerListe) :-
	spielfeld(X),member(gewinner(GewinnerListe),X).

% Die Farben und ihre dazugehoerige Position
farbe(rot,1,0,0,0,0).
farbe(gruen,0,1,0,0,0).
farbe(gelb,0,0,1,0,0).
farbe(blau,0,0,0,1,0).
farbe(pink,0,0,0,0,1).

% farben(-Liste)
% Gibt die Farben als geordnete Liste aus
farben(Liste) :- findall(Name,farbe(Name,_,_,_,_,_),Liste).

% zahl2farbe(?SpielerIDAlsZahl, ?AlsFarbe)
% Wandelt Zahlen in Farben und umgekehrt um
zahl2farbe(1, rot).
zahl2farbe(2, gruen).
zahl2farbe(3, blau).

% dt2eng(?deutscherName,?englischerName)
% Wandelt die Farbennamen in eine andere Sprache um
dt2eng(rot,red).
dt2eng(gruen,green).
dt2eng(gelb,yellow).
dt2eng(blau,blue).
dt2eng(pink,magenta).

% ???
non_unifying_delete([], _, []) :- !.
non_unifying_delete([E|T], D, R) :-
	unifiable(E, D, _), !,
	non_unifying_delete(T, D, R).
non_unifying_delete([H|T], D, [H|R]) :-
	non_unifying_delete(T, D, R).

% gibt_ritter(-Farbe)
% Zieht eine zufaelligen Farbe fuer einen Ritter
gib_ritter(Ritter) :-
	random(0, 5, Index),
	nth0(Index,[rot,gruen,gelb,blau,pink],Ritter).

% zieht Anzahl neue Ritter
ritter_nachziehen(_,0,Spielfeld,Spielfeld).
ritter_nachziehen(SpielerID,Anzahl,SpielfeldAnfang,Spielfeld) :-
	Anzahl > 0,
	gib_ritter(Ritter),
	farbe(Ritter,Ro1,Gr1,Ge1,Bl1,Pi1),
	select(reserve_ritter(SpielerID,Ro,Gr,Ge,Bl,Pi),
	       SpielfeldAnfang,
	       SpielfeldAnfang2),
	Ro2 is Ro+Ro1,
	Gr2 is Gr+Gr1,
	Ge2 is Ge+Ge1,
	Bl2 is Bl+Bl1,
	Pi2 is Pi+Pi1,
	append([reserve_ritter(SpielerID,Ro2,Gr2,Ge2,Bl2,Pi2)],
	       SpielfeldAnfang2,
	       SpielfeldEnde),
        Anzahl2 is Anzahl-1,
        ritter_nachziehen(SpielerID,Anzahl2,SpielfeldEnde,Spielfeld).

% kann_bieten(+Spieler, +Zahl)
% Vorbedingung fuer das Bieten
kann_bieten(Spieler,Zahl) :-
	spielfeld(X),
	kann_bieten(Spieler,Zahl,X).

kann_bieten(Spieler, Zahl, Spielfeld) :-
	member(bietphase([Spieler|_]),Spielfeld),
        member(noch_vorhandener_chip(Spieler, Zahl),Spielfeld),
	( \+ member(aktuelles_gebot(_,Zahl),Spielfeld) ;
	   % Chip wurde noch nicht gelegt, also ist legen moeglich
          forall((member(noch_vorhandener_chip(Spieler,Chip),Spielfeld)),member(aktuelles_gebot(_,Chip),Spielfeld))).

% moegliche_felder_d(+Start,+Schritte,-Liste)
% Gibt moegliche Felder als duplikatenfreie Liste aus
moegliche_felder_d(Start,Schritte,SListe) :-
	spielfeld(X),
	member(phase(P),X),
	P > -1,
	moegliche_felder(Start,Schritte,Liste,X),
	sort(Liste,SListe).

% moeliche_felder(+Start,+Schritte,-Liste)
% Gibt moegliche Felder als duplikaten-behaftete Liste aus
moegliche_felder(_,0,[],_).
moegliche_felder(Start,Schritte, [Nachfolger|R],Spielfeld) :-
	Schritte > 0,
	member(feld(Start,_,Nachfolger),Spielfeld),
	NSchritt is Schritte-1,
	moegliche_felder(Nachfolger,NSchritt,R,Spielfeld).

% weg2schritte(+Start,+Ziel,-Schritte)
% Start und Ziel sind Zahlen (also die FeldID's)
weg2schritte(Start,Ziel,1) :-
	feld(Start,_,Ziel).
weg2schritte(Start,Ziel,R) :-
	feld(Start,_,Nachfolger),
	Nachfolger \== Ziel,
	weg2schritte(Nachfolger,Ziel,R1),
	R is R1 + 1.

% aktueller_spieler(-Spielerfarbe)
% Gibt den aktuellen Spieler zurueck
aktueller_spieler(Spieler) :-
	spielfeld(X),
	aktueller_spieler(Spieler,X).
aktueller_spieler(Spieler,Spielfeld) :-
	member(ziehphase([Spieler|_]),Spielfeld);
	member(bietphase([Spieler|_]),Spielfeld).

% tupel2spieler(+Gebots-Spieler-Liste,-Spieler)
% Gibt fuer eine Liste von Gebots-Spieler-Tupel
% die dazugehoerigen Spieler zurueck
tupel2spieler([],[]).
tupel2spieler([_-Spieler|GT],[Spieler|R]) :- tupel2spieler(GT,R) .

% zeit(-Datumsangabe)
% Gibt eine formatierte Zeitangabe zurueck
zeit(ZeitR) :-
get_time(Zeit),
    stamp_date_time(Zeit,date(Jahr,Monat,Tag,H,M,S,_,_,_),'UTC'),

    round(S,Sr),

    atomic_list_concat([Jahr,'-',Monat,'-',Tag,'-',H,'-',M,'-',Sr], ZeitR).

kleinerFarbe(Kl,Gr) :-
	kleinerListe(Liste,Gr),
	member(Kl,Liste).

kleinerListe([],pink).
kleinerListe([Farbe2|R],Farbe) :-
	naechsteFarbe(Farbe,Farbe2),
	kleinerListe(R,Farbe2).

kleinergleichFarbe(Kl,Gr) :-
	kleinergleichListe(Liste,Gr),
	member(Kl,Liste).

kleinergleichListe([pink],pink).
kleinergleichListe([Farbe|R],Farbe) :-
	naechsteFarbe(Farbe,Farbe2),
	kleinergleichListe(R,Farbe2).

naechsteFarbe(rot,gruen).
naechsteFarbe(gruen,gelb).
naechsteFarbe(gelb,blau).
naechsteFarbe(blau,pink).

% anzahl_spieler(-Anzahl)
% Gibt die Anzahl der Spieler zurueck
anzahl_spieler(X) :-
	findall(ID,spieler(ID,_,_),Liste),
	length(Liste,X).

% phasen(?Nummer,?Bezeichnung)
% Die Bezeichnungen der einzelnen Phasen
phasen(3,'Ritter legen (1/3)').
phasen(2,'Ritter legen (2/3)').
phasen(1,'Ritter legen (3/3)').
phasen(0,'Kaiser bewegen').
phasen(-1,'Feld auswerten').
phasen(-2,'Ritter nachziehen').
phasen(-3,'Endphase').

kleinstesFeld(ID):-
	feld(ID,Vorg,_),
	Vorg > ID.

ueberpruefe_bot :-
	aktueller_spieler(Spieler),
	(bot(Spieler) -> dummerBotZug;
	not(bot(Spieler))).

:- dynamic gui_ritter_auf_feld/2,gui_kaiser/1,gui_gebot/1.

setze_einzel_akt :-
		 retractall(gui_ritter_auf_feld(_,_)),
		 retractall(gui_kaiser(_)),
		 retractall(gui_gebot(_)),
		 assert(gui_ritter_auf_feld(Farbe,FeldHof) :-
			einzel_ritter(Farbe,FeldHof)),
		 assert(gui_kaiser(Schritte) :-
			einzel_kaiser(Schritte)),
		 assert(gui_gebot(Gebot) :-
			einzel_gebot(Gebot)).

setze_netz_akt :-
		 retractall(gui_ritter_auf_feld(_,_)),
		 retractall(gui_kaiser(_)),
		 retractall(gui_gebot(_)),
		 assert(gui_ritter_auf_feld(Farbe,FeldHof) :-
			netz_ritter(Farbe,FeldHof)),
		 assert(gui_kaiser(Schritte) :-
			netz_kaiser(Schritte)),
		 assert(gui_gebot(Gebot) :-
			netz_gebot(Gebot)).

setzt_gast_akt :-
		 retractall(gui_ritter_auf_feld(_,_)),
		 retractall(gui_kaiser(_)),
		 retractall(gui_gebot(_)),
		 assert(gui_ritter_auf_feld(_,_) :- true),
		 assert(gui_kaiser(_) :- true),
		 assert(gui_gebot(_) :- true).

einzel_kaiser(Schritte) :-
	kaiser_bewegen(Schritte),
	aktualisiere_gui2.
einzel_ritter(Farbe,FeldHof) :-
	ritter_auf_feld_legen(Farbe,FeldHof),
	aktualisiere_gui2.
einzel_gebot(Gebot) :-
	gebot_eingeben(Gebot),
	aktualisiere_gui2.

netz_ritter(Farbe,FeldHof) :-
	       phase(X),
	       X > 0,
	       sende(ritter_auf_feld_legen(Farbe,FeldHof)).
netz_kaiser(Schritte) :-
	       phase(0),
	       sende(kaiser_bewegen(Schritte)).
netz_gebot(Gebot) :-
	bietphase(_),
	sende(gebot_eingeben(Gebot)).



